require 'spec_helper'

describe Authful do

  before :each do
    Authful.configure do |c|
      c.endpoint = "https://my-endpoint.dev/authful"
      c.api_key = "my-awesome-api-key"
    end
  end

  it "handles bad api key errors" do
    FakeWeb.register_uri :post, "https://my-endpoint.dev/authful/api/users", {body: {error: "Incrorrect API token"}.to_json, status: ["401", "Unauthorized"]}

    -> { Authful.enroll "john.doe@gmail.com" }.should raise_error(Authful::Errors::IncorrectApiToken)
  end

  it "creates a user" do
    FakeWeb.register_uri :post, "https://my-endpoint.dev/authful/api/users", {body: {token: "user-authful-token", email: "john.doe@gmail.com", qr_code: "qr-code"}.to_json}

    res = Authful.enroll "john.doe@gmail.com"

    res.token.should eq('user-authful-token')
    res.qr_code.should eq('qr-code')
    res.error.should be_nil
  end

  it "accepts error on duplicate user, and finds user" do
    FakeWeb.register_uri :post, "https://my-endpoint.dev/authful/api/users", {body: {error: "already enrolled", token: "user-authful-token"}.to_json, status: ["409", "Something"]}

    res = Authful.enroll "john.doe@gmail.com"

    res.token.should eq('user-authful-token')
    res.error.should eq("already enrolled")
  end

  it "errors on invalid user" do
    FakeWeb.register_uri :post, "https://my-endpoint.dev/authful/api/users", {body: {error: "Invalid user, email is required."}.to_json, status: ["400", "Bad Request"]}

    res = Authful.enroll ""

    res.error.should eq("Invalid user, email is required.")
  end

  it "view a user" do
    FakeWeb.register_uri :get, "https://my-endpoint.dev/authful/api/users/user-authful-token", {body: {ok: 1}.to_json}

    res = Authful.get_user("user-authful-token")
    res["ok"].should eq(1)
  end

  it "validates the users token" do
    FakeWeb.register_uri :get, "https://my-endpoint.dev/authful/api/users/user-authful-token/validate?token=000000", {body: {ok: 1}.to_json}

    Authful.validate("user-authful-token", "000000").should eq(true)
  end

  it "does not validate invalid tokens" do
    FakeWeb.register_uri :get, "https://my-endpoint.dev/authful/api/users/user-authful-token/validate?token=000000", {body: {error: "Invalid token"}.to_json, status: [403, "Forbidden"]}

    Authful.validate("user-authful-token", "000000").should eq(false)
  end

  it "sends sms to user" do
    FakeWeb.register_uri :get, "https://my-endpoint.dev/authful/api/users/user-authful-token/send_sms", {body: {ok: 1}.to_json}

    Authful.send_sms("user-authful-token").should eq(true)
  end

  it "sends fallback sms to user" do
    FakeWeb.register_uri :get, "https://my-endpoint.dev/authful/api/users/user-authful-token/fallback", {body: {ok: 1}.to_json}

    Authful.send_fallback_sms("user-authful-token").should eq(true)
  end

  it "catches failed sms" do
    FakeWeb.register_uri :get, "https://my-endpoint.dev/authful/api/users/user-authful-token/send_sms", {body: {error: "It just didn't send"}.to_json, status: [400, "Bad Request"]}

    Authful.send_sms("user-authful-token").should eq(false)
  end

  it "resets secret for user" do
    FakeWeb.register_uri :patch, "https://my-endpoint.dev/authful/api/users/user-authful-token/reset", {body: {token: "user-authful-token", email: "john.doe@gmail.com", qr_code: "qr-code"}.to_json}

    res = Authful.reset("user-authful-token")
    res.token.should eq('user-authful-token')
    res.qr_code.should eq('qr-code')
  end

  it "sets fallback phone number for user" do
    FakeWeb.register_uri :patch, "https://my-endpoint.dev/authful/api/users/user-authful-token/fallback", {body: {ok: 1}.to_json}

    Authful.set_fallback_phone("user-authful-token", "12005551212").should eq(true)
  end

  it "generates recovery codes" do
    FakeWeb.register_uri :post, "https://my-endpoint.dev/authful/api/users/user-authful-token/recovery_codes", {body: {ok: 1, recovery_codes: ["00000 11111 22222"]}.to_json}

    Authful.generate_recovery_codes("user-authful-token").length.should eq(1)
  end

  it "view recovery codes" do
    FakeWeb.register_uri :get, "https://my-endpoint.dev/authful/api/users/user-authful-token/recovery_codes", {body: {ok: 1, recovery_codes: ["00000 11111 22222"]}.to_json}

    Authful.generate_recovery_codes("user-authful-token").length.should eq(1)
  end

  it "validates recovery code" do
    FakeWeb.register_uri :get, "https://my-endpoint.dev/authful/api/users/user-authful-token/recovery_codes/000001111122222", {body: {ok: 1}.to_json}

    Authful.validate_recovery_code("user-authful-token", "000001111122222").should eq(true)
  end

  it "does not validate invalid recovery code" do
    FakeWeb.register_uri :get, "https://my-endpoint.dev/authful/api/users/user-authful-token/recovery_codes/000001111100000", {body: {error: "invalid recovery code"}.to_json, status: ["403", "invalid recovery code"]}

    Authful.validate_recovery_code("user-authful-token", "000001111100000").should eq(false)
    Authful.validate_recovery_code("user-authful-token", "").should eq(false)
  end

  it "deletes users" do
    FakeWeb.register_uri :delete, "https://my-endpoint.dev/authful/api/users/user-authful-token", {body: {ok: 1}.to_json}

    Authful.unenroll("user-authful-token").should eq(true)
  end

end
